Package: Rcsdp
Version: 0.1.57.6
Title: R Interface to the CSDP Semidefinite Programming Library
Authors@R: c(
    person("Hector", "Corrada Bravo", email="hcorrada@gmail.com", role=c("aut", "cre")),
    person("Florian", "Schwendinger", email="FlorianSchwendinger@gmx.at", role=c("ctb")),
    person("Brian", "Borchers", role="aut"),
    person("Don", "van den Bergh", role="ctb")
  )
Description: R interface to the CSDP semidefinite programming library. Installs version 6.1.1 of CSDP from the COIN-OR website if required. An existing installation of CSDP may be used by passing the proper configure arguments to the installation command. See the INSTALL file for further details.
LazyLoad: yes
Imports: methods
Enhances: Matrix
License: CPL-1.0
URL: https://github.com/coin-or/Csdp/
RoxygenNote: 7.0.2
BugReports: https://github.com/hcorrada/rcsdp/issues
NeedsCompilation: yes
Packaged: 2025-06-23 10:22:38 UTC; hornik
Author: Hector Corrada Bravo [aut, cre],
  Florian Schwendinger [ctb],
  Brian Borchers [aut],
  Don van den Bergh [ctb]
Maintainer: Hector Corrada Bravo <hcorrada@gmail.com>
Repository: CRAN
Date/Publication: 2025-06-23 10:24:35 UTC
